(ns pong.menu)

(def next-key-ms 200)

(defn decrease-next-key
  [{:keys [next-key progress] :as state}]
  (if (> next-key 0)
    (assoc state :next-key (- next-key progress))
    state))

(defn handle-menu-up
  [{:keys [menu-item-selected menu-items keyboard next-key] :as state}]
  (if (and (or (:KeyW keyboard) (:ArrowUp keyboard)) (<= next-key 0))
    (merge state
           {:menu-item-selected (mod (dec (+ menu-item-selected (count menu-items)))
                                     (count menu-items))
            :next-key next-key-ms})
    state))

(defn handle-menu-down
  [{:keys [menu-item-selected menu-items keyboard next-key] :as state}]
  (if (and (or (:KeyS keyboard) (:ArrowDown keyboard)) (<= next-key 0))
    (merge state
           {:menu-item-selected (mod (inc menu-item-selected) (count menu-items))
            :next-key next-key-ms})
    state))

(defn handle-menu-selected
  [{:keys [menu-item-selected menu-items keyboard] :as state}]
  (if (:Enter keyboard)
    ((:action (nth menu-items menu-item-selected))
             state)
    state))

(defn draw-text-centered
  [context txt x y]
  (let [width (.-width (.measureText context txt))]
    (.fillText context txt (- x (/ width 2)) y)))

(defn draw-menu-item
  [context item selected x y]
  (draw-text-centered context
                      (str (when selected ">")
                           (:txt item)
                           (when selected "<"))
                      x
                      y))

(defn draw
  [{:keys [context menu-item-selected window-width window-height menu-items] :as state}]
  (.clearRect context 0 0
              window-width
              window-height)

  (set! (.-fillStyle context) "#FFF")
  (set! (.-font context) "30px monospace")

  (draw-text-centered context "Pariatech's Pong" (/ window-width 2) 60)
  

  (let [middle-horizontally (/ window-width 2)
        offset-vertically (/ (- window-height (* (count menu-items) 40)) 2)]
    (doall (map-indexed #(draw-menu-item context
                                         %2
                                         (= %1 (Math/floor menu-item-selected))
                                         middle-horizontally
                                         (+ offset-vertically (* %1 40)))
                        menu-items)))
  state)

(defn scene
  [state]
  (-> state 
      decrease-next-key
      handle-menu-up
      handle-menu-down
      handle-menu-selected
      draw))
